@@ -1,7 +1,7 @@
 #
-# remt - reMarkable tablet tools
+# remt - reMarkable tablet command-line tools
 #
-# Copyright (C) 2018 by Artur Wroblewski <wrobell@riseup.net>
+# Copyright (C) 2018-2019 by Artur Wroblewski <wrobell@riseup.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,10 +18,10 @@
 #
 
 from .drawer import draw_context, draw
-from .parser import parse
+from .parser import parse, empty_page
 
-__version__ = '0.0.4'
+__version__ = '0.6.0'
 
-__all__ = ['draw_context', 'draw', 'parse', '__version__']
+__all__ = ['draw_context', 'draw', 'parse', 'empty_page', '__version__']
 
 # vim: sw=4:et:ai
