@@ -1,7 +1,7 @@
 #
-# remt - reMarkable tablet tools
+# remt - reMarkable tablet command-line tools
 #
-# Copyright (C) 2018 by Artur Wroblewski <wrobell@riseup.net>
+# Copyright (C) 2018-2019 by Artur Wroblewski <wrobell@riseup.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,18 +21,25 @@ from collections import namedtuple
 
 Page = namedtuple('Page', ['number'])
 PageEnd = namedtuple('PageEnd', ['number'])
+
 Layer = namedtuple('Layer', ['number'])
 Stroke = namedtuple(
     'Stroke',
     ['number', 'pen', 'color', 'width', 'segments']
 )
-Segment = namedtuple('Segment', ['number', 'x', 'y', 'pressure', 'tilt'])
+Segment = namedtuple(
+    'Segment',
+    ['number', 'x', 'y', 'speed', 'direction', 'width', 'pressure'],
+)
 
 Style = namedtuple(
     'Style',
-    ['width', 'color', 'join', 'cap', 'brush', 'draw_line']
+    ['color', 'join', 'cap', 'brush', 'tool_line']
 )
 Color = namedtuple('Color', ['red', 'green', 'blue', 'alpha'])
-Context = namedtuple('Context', ['cr_surface', 'cr_ctx', 'pdf_doc'])
+Context = namedtuple(
+    'Context',
+    ['cr_surface', 'cr_ctx', 'pdf_doc', 'page_number']
+)
 
 # vim: sw=4:et:ai
