@@ -1,7 +1,7 @@
 #
-# remt - reMarkable tablet tools
+# remt - reMarkable tablet command-line tools
 #
-# Copyright (C) 2018 by Artur Wroblewski <wrobell@riseup.net>
+# Copyright (C) 2018-2019 by Artur Wroblewski <wrobell@riseup.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,28 +21,21 @@
 reMarkable strokes drawing using Cairo library.
 """
 
-import gi
-gi.require_version('Poppler', '0.18')
-
 import cairo
 import io
+import itertools
 import logging
 import os.path
-import pathlib
 import pkgutil
-from collections import namedtuple
 from contextlib import contextmanager
 from functools import singledispatch, lru_cache, partial
-from gi.repository import Poppler
 
+from . import const, tool
 from .data import *
-from .line import draw_line_single, draw_line_multi
+from .pdf import pdf_open, pdf_scale
 
 logger = logging.getLogger(__name__)
 
-WIDTH = 1404
-HEIGHT = 1872
-
 COLOR_STROKE = {
     0: Color(0, 0, 0, 1),
     1: Color(0.5, 0.5, 0.5, 1),
@@ -52,83 +45,72 @@ COLOR_STROKE = {
 COLOR_HIGHLIGHTER = Color(1.0, 0.8039, 0.0, 0.1)
 
 STYLE_DEFAULT = Style(
-    1,
-    COLOR_STROKE[0],
+    None,
     cairo.LINE_JOIN_ROUND,
     cairo.LINE_CAP_ROUND,
     None,
-    draw_line_single,
+    tool.single_line,
 )
 
 STYLE_HIGHLIGHTER = Style(
-    30,
     COLOR_HIGHLIGHTER,
     cairo.LINE_JOIN_ROUND,
     cairo.LINE_CAP_SQUARE,
     None,
-    draw_line_single,
+    tool.line_highlighter,
 )
 
 STYLE_ERASER = Style(
-    None,
     COLOR_STROKE[2],
     cairo.LINE_JOIN_ROUND,
     cairo.LINE_CAP_ROUND,
     None,
-    draw_line_single,
+    tool.line_eraser,
 )
 
+style_default = STYLE_DEFAULT._replace
 
 STYLE = {
-# brush, pencil tilt and marker - tilt dependent
-#0: lambda st: Style(
-#    (5 * st.tilt) * (6 * st.width - 10) * (1 + 2 * st.pressure ** 3),
-#    COLOR_STROKE[st.color],
-#    cairo.LINE_JOIN_ROUND,
-#    cairo.LINE_CAP_ROUND,
-#),
-#1: lambda st: Style(
-#    (10 * st.tilt - 2) * (8 * st.width - 14),
-#    COLOR_STROKE[st.color]._replace(alpha=(st.pressure - 0.2) ** 2),
-#    cairo.LINE_JOIN_ROUND,
-#    cairo.LINE_CAP_ROUND,
-#),
-    # Ballpoint
-    2: lambda st: STYLE_DEFAULT._replace(
-        width=32 * st.width ** 2 - 116 * st.width + 107,
-        color=COLOR_STROKE[st.color],
-        draw_line=draw_line_multi,
-    ),
-    # Fineliner
-    4: lambda st: STYLE_DEFAULT._replace(
-            width=32 * st.width ** 2 - 116 * st.width + 107,
-            color=COLOR_STROKE[st.color],
-    ),
+    # Brush, pencil tilt and marker - tilt dependent
+    # 0: lambda st: Style(
+    #     (5 * st.tilt) * (6 * st.width - 10) * (1 + 2 * st.pressure ** 3),
+    #     COLOR_STROKE[st.color],
+    #     cairo.LINE_JOIN_ROUND,
+    #     cairo.LINE_CAP_ROUND,
+    # ),
+    # Tilt pencil
+    # 1: lambda st: STYLE_DEFAULT._replace(
+    #     width=(8 * st.width - 14),
+    #     color=COLOR_STROKE[st.color],
+    #     cap=cairo.LINE_CAP_BUTT,
+    #     brush='pencil.png',
+    #     tool_line=tool.multi_line,
+    # ),
     # Marker (tilt dependent)
-#   3: lambda st: Style(
-#       64 * st.width - 112,
-#       COLOR_STROKE[st.color],
-#       cairo.LINE_JOIN_ROUND,
-#       cairo.LINE_CAP_ROUND,
-#       None,
-#   ),
+    # 3: lambda st: STYLE_DEFAULT._replace(
+    #     width=64 * st.width - 112,
+    #     color=COLOR_STROKE[st.color],
+    #     brush=None,
+    #     tool_line=tool.multi_line,
+    # ),
+
+    # Ballpoint
+    2: style_default(tool_line=tool.line_ballpoint),
+
+    # Fineliner
+    4: style_default(tool_line=tool.line_fineliner),
+
     # Highlighter
-    5: lambda st: STYLE_HIGHLIGHTER,
+    5: STYLE_HIGHLIGHTER,
+
     # Eraser
-    6: lambda st: STYLE_ERASER._replace(
-        width=1280 * st.width ** 2 - 4800 * st.width + 4510,
-    ),
-    # Pencil - Sharp
-    7: lambda st: STYLE_DEFAULT._replace(
-        width=16 * st.width - 27,
-        color=COLOR_STROKE[st.color],
-        brush='pencil.png',
-    ),
+    6: STYLE_ERASER,
+
+    # Sharp pencil
+    7: style_default(tool_line=tool.line_sharp_pencil, brush='pencil.png'),
+
     # Erase area
-    8: lambda st: STYLE_DEFAULT._replace(
-        width=st.width,
-        color=COLOR_STROKE[st.color]._replace(alpha=0),
-    ),
+    8: STYLE_ERASER._replace(tool_line=tool.line_erase_area),
 }
 
 
@@ -149,12 +131,13 @@ def draw(item, context):
 @draw.register(Page)
 def _(page, context):
     surface = context.cr_surface
-    if page.number:
+    page_number = next(context.page_number)
+    if page_number:
         surface.show_page()
 
     if context.pdf_doc:
         # get page and set size of the current page of the cairo surface
-        pdf_page = context.pdf_doc.get_page(page.number)
+        pdf_page = context.pdf_doc.get_page(page_number)
         w, h = pdf_page.get_size()
         surface.set_size(w, h)
 
@@ -164,7 +147,7 @@ def _(page, context):
 
         # render remarkable lines data at scale to fit the document
         cr.save()  # to be restored at page end
-        factor = max(w / WIDTH, h / HEIGHT)
+        factor = pdf_scale(pdf_page)
         cr.scale(factor, factor)
 
 @draw.register(PageEnd)
@@ -178,19 +161,22 @@ def _(layer, context):
 
 @draw.register(Stroke)
 def _(stroke, context):
-    f = STYLE.get(stroke.pen)
-    if f is not None:
-        style = f(stroke)
-    else:
+    style = STYLE.get(stroke.pen)
+    if not style:
         logger.debug('Not supported pen for stroke: {}'.format(stroke))
         return
 
-    # on new path, the position of point is undefined and first `line_to`
-    # call acts as `move_to`
+    # if no predefined style color, then use stroke color
+    assert stroke.color in (0, 1, 2)
+    color = style.color
+    color = COLOR_STROKE[stroke.color] if color is None else color
+
     cr = context.cr_ctx
     cr.save()
 
-    cr.set_source_rgba(*style.color)
+    draw_stroke = partial(draw_fill, cr) if stroke.pen == 8 else cr.stroke
+
+    cr.set_source_rgba(*color)
     cr.set_line_join(style.join)
     cr.set_line_cap(style.cap)
 
@@ -198,23 +184,47 @@ def _(stroke, context):
         brush = load_brush(style.brush)
         cr.set_source(brush)
 
-    style.draw_line(cr, stroke, style)
+    lines = style.tool_line(stroke)
+    draw_multi_line(cr, draw_stroke, lines)
 
     cr.restore()
 
+def draw_multi_line(cr, draw_stroke, lines):
+    """
+    Draw multiple lines.
+
+    A line is tuple
+
+    - width of a line
+    - collection of points
+
+    :param cr: Cairo context.
+    :param draw_stroke: Drawing stroke function (i.e. line or filled area).
+    :param lines: Collection of lines to draw.
+    """
+    for width, points in lines:
+        # on new path, the position of point is undefined and first
+        # `line_to` call acts as `move_to`
+        cr.new_path()
+        cr.set_line_width(width)
+        for x, y in points:
+            cr.line_to(x, y)
+        draw_stroke()
+
+def draw_fill(cr):
+    """
+    Draw Cairo shape and fill.
+    """
+    cr.close_path()
+    cr.fill()
+
 @contextmanager
 def draw_context(fn_pdf, fn_out):
-    factor = 1
-
-    pdf_doc = None
-    if fn_pdf:
-        pdf_path = pathlib.Path(fn_pdf).resolve().as_uri()
-        pdf_doc = Poppler.Document.new_from_file(pdf_path)
-
-    surface = cairo.PDFSurface(fn_out, WIDTH, HEIGHT)
+    pdf_doc = pdf_open(fn_pdf) if fn_pdf else None
+    surface = cairo.PDFSurface(fn_out, const.PAGE_WIDTH, const.PAGE_HEIGHT)
     try:
         cr_ctx = cairo.Context(surface)
-        context = Context(surface, cr_ctx, pdf_doc)
+        context = Context(surface, cr_ctx, pdf_doc, itertools.count())
         yield context
     finally:
         surface.finish()
